﻿#nullable enable

using System;
using System.IO;
using Microsoft.Extensions.Logging;
using Microsoft.Extensions.Logging.Abstractions;

namespace Unity.Extensions.Logging
{
    public class UnityLogFormatter
    {
        private const string LogLevelPadding = ": ";

        public void Write<TState>(in LogEntry<TState> logEntry, TextWriter textWriter)
        {
            var message = logEntry.Formatter(logEntry.State, logEntry.Exception);
            if (logEntry.Exception == null && message == null)
            {
                return;
            }

            var logLevel = logEntry.LogLevel;
            var logLevelString = GetLogLevelString(logLevel);

            const string timeStampFormat = "yyyy/MM/dd HH:mm:ss.ff";
            var timeStamp = DateTimeOffset.UtcNow.ToString(timeStampFormat);
            
            textWriter.Write(timeStamp);
            textWriter.Write(' ');
            textWriter.Write('[');
            textWriter.Write(logLevelString);
            textWriter.Write(']');

            var eventId = logEntry.EventId.Id;
            var exception = logEntry.Exception;
            
            textWriter.Write(LogLevelPadding);
            textWriter.Write(logEntry.Category);
            textWriter.Write('[');
            textWriter.Write(eventId.ToString());
            textWriter.Write(']');

            WriteMessage(textWriter, message);
            
            if (exception != null)
            {
                WriteMessage(textWriter, exception.ToString());
            }
            
            textWriter.Write(Environment.NewLine);
        }

        private static void WriteMessage(TextWriter textWriter, string message)
        {
            if (string.IsNullOrEmpty(message))
            {
                return;
            }
            
            textWriter.Write(' ');
            textWriter.Write(message);
        }

        private static string GetLogLevelString(LogLevel logLevel)
        {
            return logLevel switch
            {
                LogLevel.Trace => "TRACE",
                LogLevel.Debug => "DEBUG",
                LogLevel.Information => "INFO",
                LogLevel.Warning => "WARN",
                LogLevel.Error => "ERROR",
                LogLevel.Critical => "FATAL",
                LogLevel.None => string.Empty,
                _ => throw new ArgumentOutOfRangeException(nameof(logLevel), logLevel, null)
            };
        }
    }
}