﻿#nullable enable

using System;
using System.IO;
using Microsoft.Extensions.Logging;
using Microsoft.Extensions.Logging.Abstractions;
using UnityEngine;
using ILogger = Microsoft.Extensions.Logging.ILogger;

namespace Unity.Extensions.Logging
{
    public class UnityLogger : ILogger
    {
        private readonly string _name;
        private readonly UnityEngine.ILogger _logger;
        private static StringWriter? _stringWriter;

        public UnityLogger(string name, UnityEngine.ILogger logger, UnityLogFormatter formatter)
        {
            _name = name;
            _logger = logger;
            Formatter = formatter;
        }

        internal UnityLogFormatter Formatter { get; }

        public void Log<TState>(LogLevel logLevel, EventId eventId, TState state, Exception? exception, Func<TState, Exception?, string> formatter)
        {
            if (!IsEnabled(logLevel))
            {
                return;
            }

            _stringWriter ??= new StringWriter();
            LogEntry<TState> logEntry = new(logLevel, _name, eventId, state, exception, formatter);
            Formatter.Write(in logEntry, _stringWriter);

            var sb = _stringWriter.GetStringBuilder();
            if (sb.Length == 0)
            {
                return;
            }

            var str = sb.ToString();
            sb.Clear();

            if (sb.Capacity > 1024)
            {
                sb.Capacity = 1024;
            }

            if (exception is not null)
            {
                _logger.LogException(exception);
            }
            
            if (logLevel is LogLevel.None)
            {
                return;
            }
            
            var logType = logLevel switch
            {
                LogLevel.Trace => LogType.Log,
                LogLevel.Debug => LogType.Log,
                LogLevel.Information => LogType.Log,
                LogLevel.Warning => LogType.Warning,
                LogLevel.Error => LogType.Error,
                LogLevel.Critical => LogType.Error,
                _ => throw new ArgumentOutOfRangeException(nameof(logLevel), logLevel, null)
            };
            
            _logger.Log(logType, str);
        }

        public bool IsEnabled(LogLevel logLevel)
        {
            if (!_logger.logEnabled)
            {
                return false;
            }

            return logLevel switch
            {
                LogLevel.Trace or LogLevel.Debug or LogLevel.Information => _logger.IsLogTypeAllowed(LogType.Log),
                LogLevel.Warning => _logger.IsLogTypeAllowed(LogType.Warning),
                LogLevel.Error or LogLevel.Critical => _logger.IsLogTypeAllowed(LogType.Error),
                LogLevel.None => false,
                _ => throw new ArgumentOutOfRangeException(nameof(logLevel), logLevel, null)
            };
        }

        public IDisposable BeginScope<TState>(TState state) where TState : notnull => default!;
    }
}